@@ -5,4 +5,4 @@ import unittest
 class TestWindowsFrontend(unittest.TestCase):
     def test_frontend(self):
         backend = WindowsBackend()
-        b = WinuiFrontend(backend)
+        WinuiFrontend(backend)
