@@ -12,12 +12,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import getopt
 import os
 import stat
 import sys
 import shutil
-from usbcreator.misc import popen, USBCreatorProcessException, fs_size
+from usbcreator.misc import (
+    USBCreatorProcessException,
+    callable,
+    fs_size,
+    popen,
+    )
 from usbcreator.remtimest import RemainingTimeEstimator
 from threading import Thread, Event
 import logging
@@ -26,7 +30,6 @@ from hashlib import md5
 if sys.platform != 'win32':
     from usbcreator.misc import MAX_DBUS_TIMEOUT
 
-import time
 
 class progress(Thread):
     def __init__(self, start_free, to_write, device):
@@ -53,7 +56,7 @@ class progress(Thread):
                 if callable(self.progress):
                     self.progress(v, est[0], est[1])
                 self._stopevent.wait(2)
-        except StandardError:
+        except Exception:
             logging.exception('Could not update progress:')
 
     def join(self, timeout=None):
@@ -62,13 +65,15 @@ class progress(Thread):
 
 class install(Thread):
     def __init__(self, source, target, persist, device=None,
-                 allow_system_internal=False):
+                 allow_system_internal=False,
+                 fastboot_mode=False):
         Thread.__init__(self)
         self.source = source
         self.target = target
         self.persist = persist
         self.device = device
         self.allow_system_internal = allow_system_internal
+        self.fastboot_mode = fastboot_mode
         self._stopevent = Event()
         self.progress_thread = None
         logging.debug('install thread source: %s' % source)
@@ -133,8 +138,14 @@ class install(Thread):
                 ext = os.path.splitext(self.source)[1].lower()
                 if ext not in ['.iso', '.img']:
                     self._failure(_('The extension "%s" is not supported.') %
-                                    extension)
-                if ext == '.iso':
+                                    ext)
+                if self.fastboot_mode:
+                    self.bootimg = os.path.splitext(self.source)[0] + '.bootimg'
+                    if not os.path.isfile(self.bootimg):
+                        self._failure(_('Missing matching "%s" for source image %s.') %
+                                    (self.bootimg, self.source))
+                    self.fastboot_install()
+                elif ext == '.iso':
                     if sys.platform == 'win32':
                         self.cdimage_install()
                     else:
@@ -144,7 +155,7 @@ class install(Thread):
             else:
                 self.install()
             self._success()
-        except StandardError, e:
+        except Exception as e:
             # TODO evand 2009-07-25: Bring up our own apport-like utility.
             logging.exception('Exception raised:')
             self._failure(_('An uncaught exception was raised:\n%s') % str(e))
@@ -187,6 +198,32 @@ class install(Thread):
         if os.path.exists(ldlinux):
             os.remove(ldlinux)
 
+    def os_vers(self):
+        """Return a tuple of the target OS version and our OS version."""
+        import lsb_release
+        try:
+            from debian import debian_support
+        except ImportError:
+            from debian_bundle import debian_support
+
+        target_os_ver = None
+        our_os_ver = debian_support.Version(
+            lsb_release.get_distro_information()['RELEASE'])
+
+        if os.path.exists(os.path.join(self.target, '.disk', 'info')):
+            with open(os.path.join(self.target, '.disk', 'info'),'r') as f:
+                contents = f.readline().split()
+            if len(contents) > 2:
+                # Consider point releases the same as the initial release
+                # (10.04.4 -> 10.04)
+                target_os_ver = debian_support.Version(contents[1])
+
+        return target_os_ver, our_os_ver
+
+    def need_syslinux_legacy(self):
+        target_os_ver, our_os_ver = self.os_vers()
+        return our_os_ver >= '10.10' and target_os_ver <= '10.04'
+
     def install_bootloader(self, grub_location=''):
         logging.debug('install_bootloader')
         self.progress_pulse()
@@ -211,6 +248,7 @@ class install(Thread):
                                      '/com/ubuntu/USBCreator')
                 obj.InstallBootloader(self.device, self.allow_system_internal,
                                       grub_location,
+                                      self.need_syslinux_legacy(),
                                       dbus_interface='com.ubuntu.USBCreator',
                                       timeout=MAX_DBUS_TIMEOUT)
             except dbus.DBusException:
@@ -227,7 +265,7 @@ class install(Thread):
                     os.path.join(self.target, 'syslinux'))
             os.renames(os.path.join(self.target, 'syslinux', 'isolinux.cfg'),
                     os.path.join(self.target, 'syslinux', 'syslinux.cfg'))
-        except (OSError, IOError), e:
+        except (OSError, IOError) as e:
             # Failure here probably means the source was not really an Ubuntu
             # image and did not have the files we wanted to move, see
             # <https://bugs.launchpad.net/launchpad-code/+bug/513432>
@@ -238,24 +276,11 @@ class install(Thread):
         
         # Mangle the configuration files based on the options we've selected.
         import glob
-        import lsb_release
-        try:
-            from debian import debian_support
-        except ImportError:
-            from debian_bundle import debian_support
         for filename in glob.iglob(os.path.join(self.target, 'syslinux', '*.cfg')):
             if os.path.basename(filename) == 'gfxboot.cfg':
                 continue
             f = None
-            target_os_ver = None
-            our_os_ver = debian_support.Version(
-                lsb_release.get_distro_information()['RELEASE'])
-
-            if os.path.exists(os.path.join(self.target, '.disk', 'info')):
-                with open(os.path.join(self.target, '.disk', 'info'),'r') as f:
-                    contents = f.readline().split()
-                if len(contents) > 2:
-                    target_os_ver = debian_support.Version(contents[1])
+            target_os_ver, our_os_ver = self.os_vers()
             try:
                 f = open(filename, 'r')
                 label = ''
@@ -278,19 +303,11 @@ class install(Thread):
                         #shipping in Ubuntu 10.10
                         elif (target_os_ver and our_os_ver and
                               target_os_ver != our_os_ver):
-                            lucid = debian_support.Version('10.04')
-                            maverick = debian_support.Version('10.10')
                             #10.10 or newer image, burning on 10.04 or lower
                             if (command.lower() == 'ui' and
-                                our_os_ver <= lucid and
-                                target_os_ver >= maverick):
+                                our_os_ver <= '10.04' and
+                                target_os_ver >= '10.10'):
                                 line.remove('ui')
-                            #10.04 or earlier image, burning on 10.10 or higher
-                            #Currently still broke.
-                            #elif (command.lower() == 'gfxboot' and
-                            #      our_os_ver >= maverick and
-                            #      target_os_ver <= lucid):
-                            #    line.insert(0, 'ui')
 
                     to_write.append(' '.join(line) + '\n')
                 f.close()
@@ -385,6 +402,31 @@ class install(Thread):
             except dbus.DBusException:
                 self._failure(failure_msg)
 
+    def _fastboot_command(self, cmd):
+        fastboot_cmd = ['fastboot', '-s', self.target]
+        fastboot_cmd.extend(cmd)
+        popen(fastboot_cmd)
+
+    def fastboot_install(self):
+        self.progress(0, 3*60+9, True)
+        self.progress_message(_('Erasing boot partition...'))
+        self._fastboot_command(['erase', 'boot'])
+        self.progress(5, 3*60+7, True)
+        self.progress_message(_('Erasing user partition...'))
+        self._fastboot_command(['erase', 'userdata'])
+        self.progress(10, 3*60+5, True)
+        self.progress_message(_('Flashing boot partition...'))
+        self._fastboot_command(['flash', 'boot', self.bootimg])
+        self.progress(15, 3*60+3, True)
+        self.progress_message(_('Flashing user partition...'))
+        self.progress_pulse()
+        self._fastboot_command(['flash', 'userdata', self.source])
+        self.progress_pulse_stop()        
+        self.progress(95, 1, True)
+        self.progress_message(_('Rebooting device...'))
+        self._fastboot_command(['reboot'])
+        self.progress(100, 0, True)
+
     def cdimage_install(self):
         # Build.
 
@@ -431,7 +473,7 @@ class install(Thread):
                 md5sum, filename = line.split()
                 filename = os.path.normpath(filename[2:])
                 md5sums[filename] = md5sum
-        except StandardError:
+        except Exception:
             logging.error('Could not generate the md5sum list from md5sum.txt.')
 
         self.progress_message(_('Copying files...'))
@@ -638,7 +680,7 @@ class install(Thread):
                     if callable(self.retry):
                         response = self.retry(_('Checksums do not match.  Retry?'))
                     else:
-                        respose = False
+                        response = False
                     if not response:
                         self._failure(_('Checksums do not match.'))
                 else:
